//================================================================================================
// $Id: impincwriter.h 6308 2023-01-16 20:16:41Z aron $
//
// Project:
//		Welltype Compiler
// Contents:
//		Import include writer
// Authors:
//		Aron Barath, 2014-2023
//================================================================================================
// Copyright (c) Aron Barath
//================================================================================================

#ifndef WTC_IMPINCWRITER_H
#define WTC_IMPINCWRITER_H

#include <string>
#include <fstream>
#include <set>

namespace wtast
{
class Program;
class FunctionSignature;
class RecordType;
class DataType;
class EnumType;
class OperatorDeclaration;
class FunctionType;
} // wtast

namespace wtc
{
class ProblemHandler;
} // wtc

namespace wtc
{
namespace codegen
{

class ImportIncludeWriter
{
	wtc::ProblemHandler* mProblemHandler;

	wtast::Program* mProgram;
	std::string     mSourceName;
	std::string     mFilename;
	std::ofstream   mOutFile;
	bool            mIsOpenFailed;

	std::set<std::string> mOperators;

	bool IsOpen();
	void Once(std::ostream & out);
	void WriteFunctionSignature(std::ostream & os, wtast::FunctionSignature* sig);

public:
	ImportIncludeWriter(wtc::ProblemHandler* const probhan,
		wtast::Program* const program, const std::string & sourcename,
		const std::string & filename);
	~ImportIncludeWriter();

	void WriteFunction(wtast::FunctionSignature* sig);
	void WriteException(const std::string & excname);
	void WriteRecord(wtast::RecordType* rec);
	void WriteDataType(wtast::DataType* dt);
	void WriteEnumType(wtast::EnumType* et);
	void WriteFunctionType(wtast::FunctionType* ft);
	void WriteOperator(wtast::OperatorDeclaration* op);
};

} // codegen
} // wtc

#endif // WTC_IMPINCWRITER_H
