/* $Id: ast_functionpointercall.cpp 6310 2023-01-16 20:39:48Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "ast_functionpointercall.h"
#include "ast_functiontype.h"
#include "ast_functionsignature.h"
#include "ast_expression.h"
#include "ast_simultaneoustype.h"
#include "ast_type.h"
#include "../astvisitor.h"

wtast::FunctionPointerCall::FunctionPointerCall(FunctionType* functype, Expression* funcptraccess, Expression* argexpr)
	: Expression()
	, mFuncType(functype)
	, mFuncPtrAccess(funcptraccess)
	, mArgumentExpression(argexpr)
	, mTypeInfo(0)
	, mIsTypeInfoAllocated(false)
{
	wtast::FunctionSignature* funcsig = functype->GetFunctionSignature();

	if(1==funcsig->GetRetc())
	{
		mTypeInfo = funcsig->GetRet(0);
	}
	else
	if(1<funcsig->GetRetc())
	{
		mIsTypeInfoAllocated = true;
		wtast::SimultaneousType* st = new wtast::SimultaneousType();
		mTypeInfo = st;

		for(wt_u32 i=0,n=funcsig->GetRetc();i<n;++i)
		{
			st->AddType(funcsig->GetRet(i));
		}
	}
}

wtast::FunctionPointerCall::~FunctionPointerCall()
{
	delete mFuncPtrAccess;
	delete mArgumentExpression;

	if(mIsTypeInfoAllocated)
	{
		delete mTypeInfo;
	}
}

void wtast::FunctionPointerCall::Accept(AstVisitor* vis)
{
	if(vis->Visit(this))
	{
		mFuncPtrAccess->Accept(vis);
		if(mArgumentExpression) { mArgumentExpression->Accept(vis); }

		vis->EndVisit(this);
	}
}

wtast::FunctionSignature* wtast::FunctionPointerCall::GetFunctionSignature()
{
	return mFuncType->GetFunctionSignature();
}

wtast::FunctionPointerCall* wtast::FunctionPointerCall::Copy()
{
	return new FunctionPointerCall(mFuncType, mFuncPtrAccess->Copy(), mArgumentExpression->Copy());
}
