/* $Id: ast_numberliteral.cpp 6310 2023-01-16 20:39:48Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "ast_numberliteral.h"
#include "ast_type.h"
#include "../astvisitor.h"
#include <sstream>
#include <cstring>

wtast::NumberLiteral::NumberLiteral(Type* type, bool isint, bool issig, bool is64, wt_u64 data)
	: Expression()
	, mType(type)
	, mIsInteger(isint)
	, mIsSigned(issig)
	, mIs64Bit(is64)
	, mU64(data)
{
	// ok
}

wtast::NumberLiteral::~NumberLiteral()
{
}

void wtast::NumberLiteral::Accept(AstVisitor* vis)
{
	vis->Visit(this);
}

wtast::NumberLiteral* wtast::NumberLiteral::Copy()
{
	return new NumberLiteral(mType, mIsInteger, mIsSigned, mIs64Bit, mU64);
}

std::string wtast::NumberLiteral::ToString() const
{
	std::ostringstream ss;

	if(mIsInteger)
	{
		if(mIsSigned)
		{
			if(mIs64Bit)
			{
				ss << mI64;
			}
			else
			{
				ss << mI32;
			}
		}
		else
		{
			if(mIs64Bit)
			{
				ss << mU64;
			}
			else
			{
				ss << mU32;
			}
		}
	}
	else
	{
		if(mIs64Bit)
		{
			ss << mF64;
		}
		else
		{
			ss << mF32;
		}
	}

	return ss.str();
}

wtast::TypeInfo* wtast::NumberLiteral::GetType()
{
	return static_cast<TypeInfo*>(mType);
}
