/* $Id: ast_recordupdate.cpp 6504 2023-04-03 17:46:07Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "ast_recordupdate.h"
#include "ast_recordtype.h"
#include "ast_fieldassign.h"
#include "../astvisitor.h"
#include "../../wtruntime/types.h"

wtast::RecordUpdate::~RecordUpdate()
{
	delete mAncestor;

	while(!mAssigns.empty())
	{
		delete mAssigns.front();
		mAssigns.pop_front();
	}
}

void wtast::RecordUpdate::Accept(AstVisitor* vis)
{
	if(vis->Visit(this))
	{
		mAncestor->Accept(vis);

		for(assigns_t::iterator it=mAssigns.begin(),end=mAssigns.end();it!=end;++it)
		{
			(*it)->Accept(vis);
		}

		vis->EndVisit(this);
	}
}

bool wtast::RecordUpdate::OriginateFrom(Variable::Class varclass)
{
	if(mAncestor->OriginateFrom(varclass))
	{
		return true;
	}

	for(assigns_t::iterator it=mAssigns.begin(),end=mAssigns.end();it!=end;++it)
	{
		if((*it)->GetExpression()->OriginateFrom(varclass))
		{
			return true;
		}
	}

	return Variable::READONLY==varclass || Variable::LOCAL==varclass;
}

wtast::RecordUpdate* wtast::RecordUpdate::Copy()
{
	RecordUpdate* recupd = new RecordUpdate(mAncestor->Copy());

	for(assigns_t::iterator it=mAssigns.begin(),end=mAssigns.end();it!=end;++it)
	{
		recupd->AddAssignment((*it)->Copy());
	}

	return recupd;
}

wtast::RecordType* wtast::RecordUpdate::GetRecordType()
{
	return static_cast<wtast::RecordType*>(mAncestor->GetType()->GetType(0));
}

wtast::TypeInfo* wtast::RecordUpdate::GetType()
{
	return mAncestor->GetType();
}
