/* $Id: ast_variabledeclaration.h 6310 2023-01-16 20:39:48Z aron $ */
/*
    Welltype parser and AST
    Copyright (C) 2014-2023  Aron Barath

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#ifndef WTPARSER_AST_VARIABLEDECLARATION_H
#define WTPARSER_AST_VARIABLEDECLARATION_H

#include "../astnode.h"
#include "ast_statement.h"
#include <deque>
#include <string>

namespace wtast
{

class Type;
class VariableDeclFragment;
class Expression;

class VariableDeclaration : public Statement
{
	typedef std::deque<VariableDeclFragment*> fragments_t;
	Type*       mType;
	fragments_t mFragments;
	Expression* mInitialValue;

public:
	VariableDeclaration(Type* type)
		: Statement()
		, mType(type)
		, mInitialValue(0)
	{
		// ok
	}

	~VariableDeclaration();

	void Accept(AstVisitor* vis);

	VariableDeclaration* Copy();

	Kind GetKind() const
	{
		return VARIABLEDECLARATION;
	}

	Type* GetType()
	{
		return mType;
	}

	std::size_t GetNumberOfFragments() const
	{
		return mFragments.size();
	}

	VariableDeclFragment* GetFragment(std::size_t idx)
	{
		if(idx<mFragments.size())
		{
			return mFragments[idx];
		}
		else
		{
			return 0;
		}
	}

	void AddFragment(VariableDeclFragment* fragment)
	{
		mFragments.push_back(fragment);
	}

	void SetInitialValue(Expression* expr)
	{
		mInitialValue = expr;
	}

	Expression* GetInitialValue()
	{
		return mInitialValue;
	}
};

} // namespace wtast

#endif // WTPARSER_AST_VARIABLEDECLARATION_H

