//================================================================================================
// $Id: astnode.h 6309 2023-01-16 20:19:42Z aron $
//
// Project:
//		Welltype Parser
// Contents:
//		AstNode
// Authors:
//		Aron Barath, 2014-2023
//================================================================================================
// Copyright (c) Aron Barath
//================================================================================================

#ifndef WTPARSER_ASTNODE_H
#define WTPARSER_ASTNODE_H

namespace wtast
{

class AstVisitor;

class AstNode
{
public:
	enum Kind
	{
		UNKNOWN,
		PROGRAM,
		METADATA,
		MAINBLOCK,
		CONSTRUCTORBLOCK,
		DESTRUCTORBLOCK,
		DECLAREBLOCK,
		COMPOUND,
		EXCEPTIONCOMPOUND,
		FINALLY,
		EHANDLER,
		EHANDLERS,
		BASICTYPE,
		SEQTYPE,
		RECORDTYPE,
		SIMULTANEOUSTYPE,
		VARIABLEDECLARATION,
		VARIABLEDECLFRAGMENT,
		VARIABLEACCESS,
		ELEMENTACCESS,
		MEMBERACCESS,
		SIMULTANEOUSEXPRESSION,
		STRINGLITERAL,
		BOOLEANLITERAL,
		CHARACTERLITERAL,
		NUMBERLITERAL,
		ASSIGNMENT,
		FUNCTIONSIGNATURE,
		FUNCTIONDEFINITION,
		FUNCTIONARGUMENT,
		FUNCTIONDECLARATION,
		FUNCTIONCALL,
		PROCEDURECALL,
		RECORDFIELD,
		RECORDDECLARATION,
		EXCEPTIONDECLARATION,
		PRECONDITION,
		POSTCONDITION,
		ANDTHEN,
		ORELSE,
		RETURNSTATEMENT,
		OPERATORDECLARATION,
		RAISESTATEMENT,
		WHILESTATEMENT,
		DOWHILESTATEMENT,
		FORSTATEMENT,
		FOREACHSTATEMENT,
		IFSTATEMENT,
		GLOBALBLOCK,
		ASSERTSTATEMENT,
		SEQLITERAL,
		RECORDLITERAL,
		FIELDASSIGN,
		DATATYPE,
		DATATYPEDECLARATION,
		DATATYPECONS,
		DATATYPELITERAL,
		SWITCHSTATEMENT,
		DATATYPECASESTATEMENT,
		DEFAULTSTATEMENT,
		FUNCTIONTYPE,
		FUNCTIONTYPEDECLARATION,
		FUNCTIONPOINTERCALL,
		MAKEFUNCTIONPOINTER,
		UNRESOLVEDFUNCTIONNAME,
		ENUMERATOR,
		ENUMTYPE,
		ENUMTYPEDECLARATION,
		ENUMCASESTATEMENT,
		ENUMLITERAL,
		RECORDUPDATE,
	};

	struct Variable
	{
		enum Class
		{
			LOCAL,    // local variable (as an origin can be temporary)
			GLOBAL,   // global variable
			FORMAL,   // function argument
			READONLY, // read only variable (the user can not modify its value)
		};
	};

public:
	AstNode() { }
	virtual ~AstNode() { }
	virtual Kind GetKind() const = 0;
	virtual void Accept(AstVisitor*) = 0;
	virtual AstNode* Copy() = 0;
};

} // namespace wtast

#endif // WTPARSER_ASTNODE_H
